(***********************************************************************)
(*                                                                     *)
(*  MINI FILE TRANSFERT SERVER                                         *)
(*                                                                     *)
(*  Copyright (c) 2006                                                 *)
(*    Philippe Wang                                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: minserver.ml,v 1.3 2006/07/02 17:05:06 phil Exp phil $ *)
let atomic =
  let m = Mutex.create () in
    fun f e ->
      let _ = Array.make 10000 10000 in
      let _ = Mutex.lock m in
      let r = f e in
      let _ = Mutex.unlock m in
	r

let serv file port addr= 
  try
    let sockaddr = Unix.ADDR_INET(addr, port) in
    let server i o =
      let f = open_in file in
      let s = String.create 8192 in
      let rec x() =
	let n = input f s 0 8192 in
	  output o s 0 n ;
	  if n <> 0 then x()
      in
	x();
	close_in f ;
	ignore(Array.make 10000 1000);
	atomic (fun () -> print_string "<FTC>" ; flush stdout) ()
    in Unix.establish_server server sockaddr
  with e -> 
    Printf.printf "Exception raised <%s>\n" (Printexc.to_string e);
    print_endline ("usage: "^Sys.argv.(0)^" addr port file")
      
let serv _ _ _ =
  for i = 0 to max_int do
    ignore(read_line ());
    atomic
      (fun () ->
	 Printf.printf "Nombre de lignes lues = %d\n" i;
	 flush stdout)
      ();
  done

let size = 30
  
let mkmatrix rows cols =
  let count = ref 1
  and last_col = cols - 1
  and m = Array.make_matrix rows cols 0 in
    for i = 0 to rows - 1 do
      let mi = m.(i) in
	for j = 0 to last_col do
      mi.(j) <- !count;
	  incr count
	done;
    done;
    m
      
let rec inner_loop k v m1i m2 j =
  if k < 0 then v
  else inner_loop (k - 1) (v + m1i.(k) * m2.(k).(j)) m1i m2 j
    
let mmult rows cols m1 m2 m3 =
  let last_col = cols - 1
  and last_row = rows - 1 in
    for i = 0 to last_row do
      let m1i = m1.(i) and m3i = m3.(i) in
	for j = 0 to last_col do
	  m3i.(j) <- inner_loop last_row 0 m1i m2 j
	done;
    done
      
let matmul n =
  let m1 = mkmatrix size size
  and m2 = mkmatrix size size
  and m3 = Array.make_matrix size size 0 in
    for i = 1 to max_int do
      mmult size size m1 m2 m3;
      atomic
	(fun () ->
	   Printf.printf "<MF %d>" i;
	   flush stdout)
	();
    done;
    mmult size size m1 m2 m3;
    Printf.printf "%d %d %d %d\n" m3.(0).(0) m3.(2).(3) m3.(3).(2) m3.(4).(4);
    flush stdout
      
let _ = 
  let file = Sys.argv.(3) in 
  let port = int_of_string Sys.argv.(2) in
  let addr = Unix.inet_addr_of_string Sys.argv.(1)  in
  let n = 1000 in
  let t1 = Thread.create (serv file port) addr 
  and t2 = Thread.create matmul n
  in
    (
      Thread.join t2;

      Thread.join t1
    )
